<?php
/**
 * WP Licenseware Public API
 *
 * @author Justin Greer Interactive, LLC
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
*/

/*
 |--------------------------------------------------------------------------
 | No direct access
 |--------------------------------------------------------------------------
 |
 | As a security measure there is no direct access allowed
 |
 */
if( !defined( 'ABSPATH' ) ){ header("HTTP/1.0 404 Not Found"); exit; }

/*
 |--------------------------------------------------------------------------
 | Simple Check
 |--------------------------------------------------------------------------
 |
 | Simple holder if a function was init from within the plugin
 |
 */
 $wpbu_is_validated = 'no';

/*
 |--------------------------------------------------------------------------
 | WPBU $wp_rewrites
 |--------------------------------------------------------------------------
 |
 | WPBU has its own need for a public API. We tell WP what to look for
 | and how to handle tasks.
 |
 */
add_filter( 'rewrite_rules_array','backupware_insert_rewrite_rules' );
add_filter( 'query_vars','backupware_insert_query_vars' );
add_action( 'wp_loaded','backupware_flush_rules' );

/*
 |--------------------------------------------------------------------------
 | Flush $wp_rewrite
 |--------------------------------------------------------------------------
 |
 | If WPBU rewrites are not present we need to flush the rules
 |
 */
function backupware_flush_rules(){
	$rules = get_option( 'rewrite_rules' );
	if ( !isset( $rules['wpbu-backupware/([^/]+)/?$'] ) || 
         !isset( $rules['wpbu-download/([^/]+)/?$'] ) ) {
		global $wp_rewrite;
	   	$wp_rewrite->flush_rules();
	}
}

/*
 |--------------------------------------------------------------------------
 | WPBU $wp_rewrite rules
 |--------------------------------------------------------------------------
 |
 | Add WPBU API rules to $wp_rewrite
 |
 */
function backupware_insert_rewrite_rules( $rules ){
	$newrules = array();
	$newrules['wpbu-backupware/([^/]+)/?$'] = 'index.php?wpbu-backupware=$matches[1]';
    $newrules['wpbu-download/([^/]+)/?$'] = 'index.php?wpbu-download=$matches[1]';
	return $newrules + $rules;
}

/*
 |--------------------------------------------------------------------------
 | Educating WP
 |--------------------------------------------------------------------------
 |
 | Tell WP what to look out for so WPBU api can be triggered
 |
 */
function backupware_insert_query_vars( $vars ){
    array_push($vars, 'wpbu-backupware');
    array_push($vars, 'wpbu-download' );
    return $vars;
}

/*
 |--------------------------------------------------------------------------
 | WPBU create backup using public API - Interface
 |--------------------------------------------------------------------------
 |
 | Triggers WPBU do to a backup is all checks are met
 | @todo add params to this for more control
 |
 */
function wpbu_backupware_plublic_api() {
    global $wp_query;
    if(isset($wp_query->query_vars['wpbu-backupware'])) {
        $license = get_query_var('wpbu-backupware');
        $account_settings = get_option('wpbu_account_settings');

        if(current_user_can( 'manage_options' )){

            // Dev Print out directory tree
            require_once( WPBU_ABSPATH . '/library/tasks/backup.php' );

            // Setup the backup
            $backup = new WPBU_Backup();
            $backup->backup();

            wp_redirect( admin_url() . 'index.php?page=wpbu_overview&message=Backup%20Created');
        }

        if($license == $account_settings['license_key']){
            wpbu_public_api_create_backup();
        }

        if($license != $account_settings['license_key'])
            wpbu_public_api_output(array('status' => 0, 'message' => 'Permission Denied'));

    }

    // Download a backup
    // This MUST be very strict and only people that are logged in and can manage edit themes will be able to 
    if(isset($wp_query->query_vars['wpbu-download'])) {
        
        if( !is_user_logged_in() || !current_user_can('manage_options') )
                return;

        $backup = get_query_var('wpbu-download');
        $backup_abspath = get_post_meta($backup, 'backup_location', true);
        $backup_name = basename( $backup_abspath );
        $backup_size = filesize( $backup_abspath );
        
        if($backup_abspath == '')
            return;

        header("Content-Type: application/zip");
        header("Content-Disposition: attachment; filename=$backup_name");
        header("Content-Length: " . $backup_size);

        readfile( $backup_abspath );
        exit;
    }
}
add_action( 'template_redirect', 'wpbu_backupware_plublic_api');

/*
 |--------------------------------------------------------------------------
 | Give WPBU a voice
 |--------------------------------------------------------------------------
 |
 | Handles the output for the API in JSON
 |
 */
function wpbu_public_api_output( $output ){
    header('Content-type: application/json');
    print json_encode( $output );
    exit;
}

/*
 |--------------------------------------------------------------------------
 | WPBU create a backup - Public API
 |--------------------------------------------------------------------------
 |
 | Grab the needed items and create a backup
 | @todo Possibly add settings checks to control wht accually happens
 |
 */
function wpbu_public_api_create_backup(){
    require_once( WPBU_ABSPATH . '/library/tasks/backup.php' );
    $backup = new WPBU_Backup();
    $backup->backup();
    wpbu_public_api_output(array('status' => 1, 'message' => 'Backup was succesfull'));
}