<?php
/*
 |--------------------------------------------------------------------------
 | Updates WPBU Shedule settings
 |--------------------------------------------------------------------------
 |
 | Before updating the schedule for backups, this validates the current 
 | license. Automated backups using WPBU services striclty for valid licenses
 |
 */
add_action('pre_update_option_wpbu_schedule_settings', 'wpbu_update_schedule');
function wpbu_update_schedule( $values ){
	$account_settings = get_option('wpbu_account_settings');
	$args = array(
	    'timeout'     => 5,
	    'redirection' => 5,
	    'httpversion' => '1.0',
	    'user-agent'  => 'WordPress/' . $wp_version . '; ' . get_bloginfo( 'url' ),
	    'blocking'    => true,
	    'headers'     => array(),
	    'cookies'     => array(),
	    'body'        => null,
	    'compress'    => false,
	    'decompress'  => true,
	    'sslverify'   => true,
	    'stream'      => false,
	    'filename'    => null
	);
	$validation = wp_remote_get( WPBU_REMOTE_HOME . '/api/update-schedule/?key='.$account_settings['license_key'].'&schedule='.$values['backup_frequency'].'&domain='.base64_encode(site_url()) , $args );
	$response = json_decode( $validation['body'] );
	if(isset($response->error)){
		$values['backup_frequency'] = "OFF";
		return $values;
	}
	return $values;
}

/*
 |--------------------------------------------------------------------------
 | Validates WPBU License
 |--------------------------------------------------------------------------
 |
 | Validates and then updates WPBU's settings appropriately 
 |
 */
add_action('pre_update_option_wpbu_account_settings', 'wpbu_license_check');
function wpbu_license_check( $values ){
	$account_settings = get_option('wpbu_account_settings');
	if(empty($values['license_key'])){
		$values['is_licensed'] 	= false;
		return $values;
	}
	$args = array(
	    'timeout'     => 5,
	    'redirection' => 5,
	    'httpversion' => '1.0',
	    'user-agent'  => 'WordPress/' . $wp_version . '; ' . get_bloginfo( 'url' ),
	    'blocking'    => true,
	    'headers'     => array(),
	    'cookies'     => array(),
	    'body'        => null,
	    'compress'    => false,
	    'decompress'  => true,
	    'sslverify'   => true,
	    'stream'      => false,
	    'filename'    => null
	);
	$validation = wp_remote_get( WPBU_REMOTE_HOME . '/api/validate-account/?key=' . $values['license_key'] , $args );
	
	//print_r($validation); exit;

	$response = json_decode( $validation['body'] );
	
	if(!$response->isValid){
		$values['is_licensed'] = false;
	}else{
		$values['is_licensed'] = true;
	}
	return $values;
}

/*
 |--------------------------------------------------------------------------
 | Checks is WordPress is locked
 |--------------------------------------------------------------------------
 |
 | When WPBU locks WordPress it creates file labeled wpbu.loc in the root
 | If the file exists and a user can not manage options, we will put WordPres 
 | into maintenance mode.
 |
 */
function wpbu_is_wp_locked(){
	if( file_exists(ABSPATH . 'wpbu.loc') && !current_user_can('edit_themes') )
		wp_die('Our site is down for maintenance. Try again in a few minutes.');
}
add_action('init', 'wpbu_is_wp_locked');

/*
 |--------------------------------------------------------------------------
 | Remove a Backup
 |--------------------------------------------------------------------------
 |
 | Removes a backup as well as removes all information. This also removes
 | the backup package from the backups as well.
 |
 */
function wpbu_remove_backup( $backupID ){

	if(empty($backupID))
		wp_die( 'BackupID is missing.' );

   	//$rel_post = get_posts('p='.$backupID);
   	//if(!$rel_post)
   	//	wp_die( 'BackupID does not exist.' );

   	// Delete the actual backup file
   	$backupPath = get_post_meta($backupID, 'backup_location', true);
   	@unlink( $backupPath );

   	// Delete the post
   	wp_delete_post( $backupID, true );

}

/*
 |--------------------------------------------------------------------------
 | Locks WordPress
 |--------------------------------------------------------------------------
 |
 | Creates file "wpbu.loc" in the root (ABSPATH)
 |
 */
function wpbu_lock_wp(){
	$lock_file = 'wpbu.loc';
	$lock = fopen(ABSPATH . $lock_file, 'w') or wp_die('Cannot open file:  ' . $lock_file);
}

/*
 |--------------------------------------------------------------------------
 | Unlocks WordPress
 |--------------------------------------------------------------------------
 |
 | Removes the file "wpbu.loc" from WordPress root
 |
 */
function wpbu_unlock_wp(){
	$lock_file = 'wpbu.loc';
	$unlock = unlink( ABSPATH . $lock_file );
}

/*
 |--------------------------------------------------------------------------
 | Permalink Check
 |--------------------------------------------------------------------------
 |
 | In order for the plugin to work, WP must have permalinks enabled
 |
 */
$wpbu_permalink_check = get_option( 'rewrite_rules' );
if(!$wpbu_permalink_check) add_action( 'admin_notices', 'wpbu_display_permalink_error' );
function wpbu_display_permalink_error(){
	print '<div class="updated error">';
	print '<p>WPBACKUPWARE requires that permalinks be set to something other than default. <a href="'.admin_url().'options-permalink.php">Change your permalink settings</a>. </p>';
	print '</div>';
}