<?php
/**
 * WPBU Awesomeness
 *
 * @author Justin Greer
 * @version 1.0.0
 *
 * STEPS
 * 1. open and create a backup directory with zipArchive.
 * 2. init database backup with php_mysqldump and label with database_bak.sql in root of new backupdirectory
 * 3. init a filesystem backup and clone directory ORG directory tree into the backup root
 * 4. save the params used for the backup using postmeta('posttype', 'key', 'value');
 * 5. return to the user that the backup has been created it not
 *
 * ALT ROUTE
 * 1. wp install requests a token from WP Backupware servers
 * 2. server generates a token valid for 30 minutes and returns the token to the wp install
 * 3. wp send data about where the backup is located. wp then also sets a timer to access the backup. 
 * This token is valid for only 30 minutes.
 * 4. The server will respond with a 'ok' message at which time the servers will gather where the members storage
 * will be located. The server will also update the users account using remote filesize. User will be alloted 1 Gig 
 * of storage. 
 */
if( !defined( 'ABSPATH' ) ){ header("HTTP/1.0 404 Not Found"); exit; }

class WPBU_Backup {

	public $version = "1.0.0";
	public $mysql_version;
	public $time;
	public $database_backup;
	public $backup_zip;
	public $start_time;
	public $end_time;

	// File System Related Stuff
	public $file_structure = array();
	public $backup_exeptions = array('backups');

	/*
	 |--------------------------------------------------------------------------
	 | Construct Method
	 |--------------------------------------------------------------------------
	 |
	 | Sets the class up for success
	 |
	 */
	public function __construct(){

		// Double tap the backup directory
		wpbu_install();

		// Setup the mackup file
		$this->time = time();
		$this->mysql_version = $this->get_mysql_version();
		$this->database_backup = WPBU_BACKUP_PATH . '/'.$this->time.'.sql';
		$this->backup_zip = WPBU_BACKUP_PATH.'/'.$this->time.'.zip';
	}

	/*
	 |--------------------------------------------------------------------------
	 | Start the backup process
	 |--------------------------------------------------------------------------
	 |
	 | The start of the backup
	 |
	 */
	public function backup(){

		// Create the zip file that will be used for the backup
		$zip = new ZipArchive();
		$zip->open( $this->backup_zip, ZipArchive::CREATE);

		// Start the database backup
		global $wpdb;
		$this->write_mysql_bak_header_information();
    	$tables = $wpdb->get_results( "SHOW TABLES" );
		$table_call = 'Tables_in_'.DB_NAME;
		foreach( $tables as $table ){
  
		  // Table name
		  $table_name = $table->$table_call;
			  
		  // Table Comment
		  $this->write( "--" );
		  $this->write( "-- Table structure for table `".$table_name."`");
		  $this->write( "--" );
		  $this->write( "");
  
		  // Drop Table
		  $this->write( "DROP TABLE IF EXISTS `".$table_name."`;");
  
		  // Write "Create Table"
		  $create_table = $this->show_create_table( $table_name );
		  $this->write( $create_table . ";" );
		  $this->write( "");
  
		  // Comment for dump content
		  $this->write( "--");
		  $this->write( "-- Dumping data for table `".$table_name."`");
			  $this->write( "--");
			  $this->write( "");
  
			  // Lock Table
			  $this->write( "LOCK TABLES `".$table_name."` WRITE;");
  
			  // There is currently no way to create  insert table
			  // I will use a custom buile method for this
			  $this->write( $this->show_create_insert( $table_name ) );
  
			  // Unlock the table and continue
			  $this->write( "UNLOCK TABLES;");
			  $this->write( "");
		  }	

		  // Add the Database backup to the backupZIP
		  $zip->addFile( $this->database_backup, '/database_backup.sql');
  
		  // Now we need to create the file system backup
		  $this->dir_tree('/');
		  foreach( $this->file_structure as $file ){
			  $zip->addFile( ABSPATH.$file, $file );
		  }

		  // Make sure the file is closed. if not the filesize will not be able to read it
		  $zip->close();

		  // Create the backup in WordPress
		  global $user_ID;
		  $page['post_type']    = 'wpbu_backups';
		  $page['post_content'] = 'WP Backupware - ' . $this->time;
		  $page['post_parent']  = 0;
		  $page['post_author']  = $user_ID;
		  $page['post_status']  = 'private';
		  $page['post_title']   = 'WP Backupware - ' . $this->time;
		  $pageid = wp_insert_post ($page);
		  if ($pageid != 0) { 

			// Insert custom meta data
			add_post_meta( $pageid, 'backup_location', $this->backup_zip);
			add_post_meta( $pageid, 'backup_type', 'Full');
	  
			@$backup_size = $this->get_file_size( $this->backup_zip, 'MB', $pageid );
			add_post_meta( $pageid, 'backup_size', $backup_size);
	  
			// Send email notification if enabled
			$wpbu_settings = get_option('wpbu_general_settings');
			if($wpbu_settings['send_email_notification'] == 'ON'){
	  
			  if($wpbu_settings['email_notification_contact'] == ''){
				$recipient = get_bloginfo('admin_email');
			  }else{
				$recipient = $wpbu_settings['email_notification_contact'];
			  }
	  
			  wp_mail( $recipient, 'Backup Notification', 'This mail is to let you know that your website has just been backed up');
			}
				  
			// Remove database SQL (clean up)
			@unlink( $this->database_backup );    
				  
			// @TODO Trigger additional FTP, Dropbox, Upload (comming soon)
			return array(
						  'backup_status' => 'Completed',
						  'backup_date' => date('F jS, Y  @ H: i: s'),
						  'backup_id' => $pageid,
						  'backup_title' => $page['post_title'],
						  'backup_size' => $backup_size,
						  'backup_type' => 'database'
						  );
	  
			}else{
				  return array(
							  'backup_status' => 'failed',
							  'error' => 'The backup could not be completed'
							  );
	  
			}

	}

	/*
	 |--------------------------------------------------------------------------
	 | Get file size
	 |--------------------------------------------------------------------------
	 |
	 */
    public function get_file_size($file, $type, $ID){
       switch($type){
          case "KB":
             $filesize = filesize($file) * .0009765625; // bytes to KB
          break;
          case "MB":
             $filesize = (filesize($file) * .0009765625) * .0009765625; // bytes to MB
          break;
          case "GB":
             $filesize = ((filesize($file) * .0009765625) * .0009765625) * .0009765625; // bytes to GB
          break;
       }
       if($filesize <= 0){
       	@add_post_meta( $ID, 'backup_status', 'Failed');
        return $filesize = 'Unknown';
       }else{
        @add_post_meta( $ID, 'backup_status', 'Completed');
        return round($filesize, 2).' '.$type;
        }
    }

	/*
	 |--------------------------------------------------------------------------
	 | Show create table syntax
	 |--------------------------------------------------------------------------
	 |
	 | Syntax straight form the MySQL server
	 |
	 */
    protected function show_create_table($table = null){
    	if(empty($table))
    		return;
    	global $wpdb;
    	$status = $wpdb->get_results("SHOW CREATE TABLE {$table}");
    	return $status[0]->{'Create Table'};
    }

    /*
	 |--------------------------------------------------------------------------
	 | Show create table syntax
	 |--------------------------------------------------------------------------
	 |
	 | Collect data and returns a formatted dump of any given table contents
	 |
	 */
    protected function show_create_insert($table = null){
    	if(empty($table))
    		return;
    	global $wpdb;
    	$data = $wpdb->get_results("SELECT * FROM {$table}");

    	$return ='';
    	if(count($data)>0){

    		foreach($data as $insert){
    			$value_array = array();
    			foreach($insert as $key=>$value){
    				if ( is_int($value) ){
    					array_push($value_array, $value);
    				}else{
    					array_push($value_array, "'".str_replace("'", "\'", $value)."'");
    				}
    			}
    			$row_values = implode(", ", $value_array);

    			// Build the first half of the insert
    			$return .= "INSERT INTO `".$table."` VALUES (".$row_values."); \n";
    			
    		}

    	}
    	return $return;
    }

	/*
	 |--------------------------------------------------------------------------
	 | Return MySQL Version
	 |--------------------------------------------------------------------------
	 |
	 | Retuns a formatted version number of current MySQL
	 |
	 */
	public function get_mysql_version(){
		global $wpdb;
		$version = $wpdb->get_results("SELECT VERSION()");
       	return $version[0]->{'VERSION()'};
	}

	/*
	 |--------------------------------------------------------------------------
	 | Write header information 
	 |--------------------------------------------------------------------------
	 |
	 | Write header information to the database backup dump
	 |
	 */
	protected function write_mysql_bak_header_information(){
		// Header Information
        $this->write( "-- PHP MySQL Dump Alpha, WP BACKUPWARE (".$this->version.")" );
        $this->write( "--" );
        $this->write( "-- Host: ".DB_HOST."    Database: ".DB_NAME."" );
        $this->write( "--------------------------------------------------------" );
        $this->write( "-- Server version  ".$this->mysql_version."" );
        $this->write( "" );
	}

	/*
	 |--------------------------------------------------------------------------
	 | Gets the file system tree
	 |--------------------------------------------------------------------------
	 |
	 | Returns the entire filesystems absolute paths
	 |
	 */
	public function dir_tree($dir){
		$relativedir = '.'.$dir;
	    if($dh = opendir($relativedir)){
		    while(false !== ($file = readdir($dh))){
		        if(($file !== '.') && ($file !== '..')){

		        	if(in_array($file,$this->backup_exeptions))
		        		continue;

		            if(!is_dir($relativedir.$file)){
		                array_push($this->file_structure, substr($dir.$file, 1));
		            }else{
		                $this->dir_tree($dir.$file.'/');
		            }
		        }
		    }
	    }
	}

	/*
	 |--------------------------------------------------------------------------
	 | Write to the file current backup file
	 |--------------------------------------------------------------------------
	 |
	 | Retuns a formatted version number of current MySQL
	 |
	 */
	protected function write( $content = null ){
    	if(empty($content))
    		return;

        $handler = fopen( $this->database_backup, 'a' );
        fwrite( $handler,  $content . "\n");
        fclose( $handler );
    }

}