<?php
/**
 * Plugin Name: WP Backupware
 * Plugin URI: http://wpbackupware.com
 * Version: 1.0.1
 * Description: The most awesome way to backup WordPress 
 * Author: wpbackupware
 * Author URI: http://wpbackupware.com
 * License: GPL2
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
*/

/*
 |--------------------------------------------------------------------------
 | No direct access
 |--------------------------------------------------------------------------
 |
 | As a security measure there is no direct access allowed
 |
 */
if( !defined( 'ABSPATH' ) ){ header("HTTP/1.0 404 Not Found"); exit; }

/*
 |--------------------------------------------------------------------------
 | WPBU plugin version 
 |--------------------------------------------------------------------------
 |
 | This is use within the plugin only any version changes must be made to
 | the header as well.
 |
 */
define( 'WPBU_VERSION', '1.0.1' );

/*
 |--------------------------------------------------------------------------
 | WPBU absolute path
 |--------------------------------------------------------------------------
 |
 | WWe could use WP's codex but lets keep in in house
 |
 */
define( 'WPBU_ABSPATH', dirname( __FILE__ ) );

/*
 |--------------------------------------------------------------------------
 | WPBU HTTP path
 |--------------------------------------------------------------------------
 |
 | This is used for includes like scripts and styles
 |
 */
define( 'WPBU_URLPATH', plugins_url( '/', __FILE__ ) );

/*
 |--------------------------------------------------------------------------
 | WPBU absolute backup path
 |--------------------------------------------------------------------------
 |
 | WPBU will store its backups here.
 | wp_upload_dir() is being used to get wp uploads path. ;)
 |
 */
$wp_upload_basedir_wpbu = wp_upload_dir();
define( 'WPBU_BACKUP_PATH', $wp_upload_basedir_wpbu['basedir'] . '/wpbu-backups' );

/*
 |--------------------------------------------------------------------------
 | WPBU home endpoint
 |--------------------------------------------------------------------------
 |
 | This is where WPBU will make it calls to for the settings sync
 |
 */
define( 'WPBU_REMOTE_HOME', 'http://wpbackupware.com' );

/*
 |--------------------------------------------------------------------------
 | WPBU activation hook for WP
 |--------------------------------------------------------------------------
 |
 | When WPBU is activated this will be triggered
 | 
 | Installs singles options (as groups)
 | 	- wpbu_account_settings
 | 	- wbu_general_settings
 |	- wpbu_schedule_settings
 |
 */
register_activation_hook( __FILE__, 'wpbu_install');
function wpbu_install() {

	// Multisite not supported yet
	if(is_multisite())
		wp_die('Looks like WordPress is using multisite. This version is the plugin does not yet support multisite.');

	// @todo add other checks here and make proper settings adjustments	
	add_option('wpbu_account_settings');
	add_option('wpbu_general_settings');
	add_option('wpbu_schedule_settings');

	// Create the backup directory. This is also checked on every backup
	// as well. The path should exisit already but we will make sure with recurs
	if( !file_exists( WPBU_BACKUP_PATH ))
		mkdir( WPBU_BACKUP_PATH, 0755 , true );

	// Protect the directory from direct access - deny from all
	if( !file_exists( WPBU_BACKUP_PATH . '/.htaccess' )){
		$htaccess = fopen( WPBU_BACKUP_PATH . '/.htaccess', 'w');
		$data = 'deny from all';
		fwrite( $htaccess, $data);
		fclose( $htaccess );
	}
}

/*
 |--------------------------------------------------------------------------
 | WPBU core hook
 |--------------------------------------------------------------------------
 |
 | Grab the core class for WPBU and init it
 |
 */
require_once( WPBU_ABSPATH . '/core.php' );
new WPBU_Core;

/*
 |--------------------------------------------------------------------------
 | WPBU functions hook
 |--------------------------------------------------------------------------
 |
 | Grab some other needed functions
 |
 */
require_once( WPBU_ABSPATH . '/functions.php' );

/*
 |--------------------------------------------------------------------------
 | WPBU api hook
 |--------------------------------------------------------------------------
 |
 | Grabs the api functions for WPBU
 |
 */
require_once( WPBU_ABSPATH . '/api.php' );

/*
 |--------------------------------------------------------------------------
 | WPBU WP post type hook
 |--------------------------------------------------------------------------
 |
 | Instead of adding on to the database we will use WP core to do the
 | heavy lifting.
 |
 */
add_action('init', array('WPBU_Core','regster_backup_postype') );


/*
 |--------------------------------------------------------------------------
 | WPBU asset register
 |--------------------------------------------------------------------------
 |
 | Registers needed assets for WPBU's admin interface
 |
 */
add_action( 'admin_init', 'wpbu_regsiter_assets' );
function wpbu_regsiter_assets(){
	wp_register_style( 'wpbu-styles', WPBU_URLPATH . 'assets/css/styles.css' );
}

/*
 |--------------------------------------------------------------------------
 | Icon for WPBU
 |--------------------------------------------------------------------------
 |
 | Icon for WP - WP 3.8 menu icons
 |
 */
function add_menu_icons_styles(){ ?>
	<style>
		#adminmenu #toplevel_page_wpbu_overview div.wp-menu-image:before {
		  content: "\f176";
		}
	</style>
 
<?php }
add_action( 'admin_head', 'add_menu_icons_styles' );